! Copyright (C) 2005, 2006 Doug Coleman.
! See http://factorcode.org/license.txt for BSD license.
USING: accessors assocs hashtables kernel math namespaces words
windows.types vocabs sequences ;
IN: windows.messages

SYMBOL: windows-messages

"windows.messages" words
[ name>> "windows-message" head? not ] filter
[ dup execute swap ] { } map>assoc
windows-messages set-global

: windows-message-name ( n -- name )
    windows-messages get at "unknown message" or ;

CONSTANT: WM_NULL HEX: 0000
CONSTANT: WM_CREATE HEX: 0001
CONSTANT: WM_DESTROY HEX: 0002
CONSTANT: WM_MOVE HEX: 0003
CONSTANT: WM_SIZE HEX: 0005
CONSTANT: WM_ACTIVATE HEX: 0006
CONSTANT: WM_SETFOCUS HEX: 0007
CONSTANT: WM_KILLFOCUS HEX: 0008
CONSTANT: WM_ENABLE HEX: 000A
CONSTANT: WM_SETREDRAW HEX: 000B
CONSTANT: WM_SETTEXT HEX: 000C
CONSTANT: WM_GETTEXT HEX: 000D
CONSTANT: WM_GETTEXTLENGTH HEX: 000E
CONSTANT: WM_PAINT HEX: 000F
CONSTANT: WM_CLOSE HEX: 0010
CONSTANT: WM_QUERYENDSESSION HEX: 0011
CONSTANT: WM_QUERYOPEN HEX: 0013
CONSTANT: WM_ENDSESSION HEX: 0016
CONSTANT: WM_QUIT HEX: 0012
CONSTANT: WM_ERASEBKGND HEX: 0014
CONSTANT: WM_SYSCOLORCHANGE HEX: 0015
CONSTANT: WM_SHOWWINDOW HEX: 0018
CONSTANT: WM_WININICHANGE HEX: 001A
CONSTANT: WM_SETTINGCHANGE HEX: 001A
CONSTANT: WM_DEVMODECHANGE HEX: 001B
CONSTANT: WM_ACTIVATEAPP HEX: 001C
CONSTANT: WM_FONTCHANGE HEX: 001D
CONSTANT: WM_TIMECHANGE HEX: 001E
CONSTANT: WM_CANCELMODE HEX: 001F
CONSTANT: WM_SETCURSOR HEX: 0020
CONSTANT: WM_MOUSEACTIVATE HEX: 0021
CONSTANT: WM_CHILDACTIVATE HEX: 0022
CONSTANT: WM_QUEUESYNC HEX: 0023
CONSTANT: WM_GETMINMAXINFO HEX: 0024
CONSTANT: WM_PAINTICON HEX: 0026
CONSTANT: WM_ICONERASEBKGND HEX: 0027
CONSTANT: WM_NEXTDLGCTL HEX: 0028
CONSTANT: WM_SPOOLERSTATUS HEX: 002A
CONSTANT: WM_DRAWITEM HEX: 002B
CONSTANT: WM_MEASUREITEM HEX: 002C
CONSTANT: WM_DELETEITEM HEX: 002D
CONSTANT: WM_VKEYTOITEM HEX: 002E
CONSTANT: WM_CHARTOITEM HEX: 002F
CONSTANT: WM_SETFONT HEX: 0030
CONSTANT: WM_GETFONT HEX: 0031
CONSTANT: WM_SETHOTKEY HEX: 0032
CONSTANT: WM_GETHOTKEY HEX: 0033
CONSTANT: WM_QUERYDRAGICON HEX: 0037
CONSTANT: WM_COMPAREITEM HEX: 0039
CONSTANT: WM_GETOBJECT HEX: 003D
CONSTANT: WM_COMPACTING HEX: 0041
CONSTANT: WM_COMMNOTIFY HEX: 0044
CONSTANT: WM_WINDOWPOSCHANGING HEX: 0046
CONSTANT: WM_WINDOWPOSCHANGED HEX: 0047
CONSTANT: WM_POWER HEX: 0048
CONSTANT: WM_COPYDATA HEX: 004A
CONSTANT: WM_CANCELJOURNAL HEX: 004B
CONSTANT: WM_NOTIFY HEX: 004E
CONSTANT: WM_INPUTLANGCHANGEREQUEST HEX: 0050
CONSTANT: WM_INPUTLANGCHANGE HEX: 0051
CONSTANT: WM_TCARD HEX: 0052
CONSTANT: WM_HELP HEX: 0053
CONSTANT: WM_USERCHANGED HEX: 0054
CONSTANT: WM_NOTIFYFORMAT HEX: 0055
CONSTANT: WM_CONTEXTMENU HEX: 007B
CONSTANT: WM_STYLECHANGING HEX: 007C
CONSTANT: WM_STYLECHANGED HEX: 007D
CONSTANT: WM_DISPLAYCHANGE HEX: 007E
CONSTANT: WM_GETICON HEX: 007F
CONSTANT: WM_SETICON HEX: 0080
CONSTANT: WM_NCCREATE HEX: 0081
CONSTANT: WM_NCDESTROY HEX: 0082
CONSTANT: WM_NCCALCSIZE HEX: 0083
CONSTANT: WM_NCHITTEST HEX: 0084
CONSTANT: WM_NCPAINT HEX: 0085
CONSTANT: WM_NCACTIVATE HEX: 0086
CONSTANT: WM_GETDLGCODE HEX: 0087
CONSTANT: WM_SYNCPAINT HEX: 0088
CONSTANT: WM_NCMOUSEMOVE HEX: 00A0
CONSTANT: WM_NCLBUTTONDOWN HEX: 00A1
CONSTANT: WM_NCLBUTTONUP HEX: 00A2
CONSTANT: WM_NCLBUTTONDBLCLK HEX: 00A3
CONSTANT: WM_NCRBUTTONDOWN HEX: 00A4
CONSTANT: WM_NCRBUTTONUP HEX: 00A5
CONSTANT: WM_NCRBUTTONDBLCLK HEX: 00A6
CONSTANT: WM_NCMBUTTONDOWN HEX: 00A7
CONSTANT: WM_NCMBUTTONUP HEX: 00A8
CONSTANT: WM_NCMBUTTONDBLCLK HEX: 00A9
CONSTANT: WM_NCXBUTTONDOWN HEX: 00AB
CONSTANT: WM_NCXBUTTONUP HEX: 00AC
CONSTANT: WM_NCXBUTTONDBLCLK HEX: 00AD
CONSTANT: WM_NCUAHDRAWCAPTION HEX: 00AE ! undocumented
CONSTANT: WM_NCUAHDRAWFRAME HEX: 00AF   ! undocumented
CONSTANT: WM_INPUT HEX: 00FF
CONSTANT: WM_KEYFIRST HEX: 0100
CONSTANT: WM_KEYDOWN HEX: 0100
CONSTANT: WM_KEYUP HEX: 0101
CONSTANT: WM_CHAR HEX: 0102
CONSTANT: WM_DEADCHAR HEX: 0103
CONSTANT: WM_SYSKEYDOWN HEX: 0104
CONSTANT: WM_SYSKEYUP HEX: 0105
CONSTANT: WM_SYSCHAR HEX: 0106
CONSTANT: WM_SYSDEADCHAR HEX: 0107
CONSTANT: WM_UNICHAR HEX: 0109
CONSTANT: WM_KEYLAST_NT501 HEX: 0109
CONSTANT: UNICODE_NOCHAR HEX: FFFF
CONSTANT: WM_KEYLAST_PRE501 HEX: 0108
CONSTANT: WM_IME_STARTCOMPOSITION HEX: 010D
CONSTANT: WM_IME_ENDCOMPOSITION HEX: 010E
CONSTANT: WM_IME_COMPOSITION HEX: 010F
CONSTANT: WM_IME_KEYLAST HEX: 010F
CONSTANT: WM_INITDIALOG HEX: 0110
CONSTANT: WM_COMMAND HEX: 0111
CONSTANT: WM_SYSCOMMAND HEX: 0112
CONSTANT: WM_TIMER HEX: 0113
CONSTANT: WM_HSCROLL HEX: 0114
CONSTANT: WM_VSCROLL HEX: 0115
CONSTANT: WM_INITMENU HEX: 0116
CONSTANT: WM_INITMENUPOPUP HEX: 0117
CONSTANT: WM_MENUSELECT HEX: 011F
CONSTANT: WM_MENUCHAR HEX: 0120
CONSTANT: WM_ENTERIDLE HEX: 0121
CONSTANT: WM_MENURBUTTONUP HEX: 0122
CONSTANT: WM_MENUDRAG HEX: 0123
CONSTANT: WM_MENUGETOBJECT HEX: 0124
CONSTANT: WM_UNINITMENUPOPUP HEX: 0125
CONSTANT: WM_MENUCOMMAND HEX: 0126
CONSTANT: WM_CHANGEUISTATE HEX: 0127
CONSTANT: WM_UPDATEUISTATE HEX: 0128
CONSTANT: WM_QUERYUISTATE HEX: 0129
CONSTANT: WM_CTLCOLORMSGBOX HEX: 0132
CONSTANT: WM_CTLCOLOREDIT HEX: 0133
CONSTANT: WM_CTLCOLORLISTBOX HEX: 0134
CONSTANT: WM_CTLCOLORBTN HEX: 0135
CONSTANT: WM_CTLCOLORDLG HEX: 0136
CONSTANT: WM_CTLCOLORSCROLLBAR HEX: 0137
CONSTANT: WM_CTLCOLORSTATIC HEX: 0138
CONSTANT: WM_MOUSEFIRST HEX: 0200
CONSTANT: WM_MOUSEMOVE HEX: 0200
CONSTANT: WM_LBUTTONDOWN HEX: 0201
CONSTANT: WM_LBUTTONUP HEX: 0202
CONSTANT: WM_LBUTTONDBLCLK HEX: 0203
CONSTANT: WM_RBUTTONDOWN HEX: 0204
CONSTANT: WM_RBUTTONUP HEX: 0205
CONSTANT: WM_RBUTTONDBLCLK HEX: 0206
CONSTANT: WM_MBUTTONDOWN HEX: 0207
CONSTANT: WM_MBUTTONUP HEX: 0208
CONSTANT: WM_MBUTTONDBLCLK HEX: 0209
CONSTANT: WM_MOUSEWHEEL HEX: 020A
CONSTANT: WM_XBUTTONDOWN HEX: 020B
CONSTANT: WM_XBUTTONUP HEX: 020C
CONSTANT: WM_XBUTTONDBLCLK HEX: 020D
CONSTANT: WM_MOUSELAST_5 HEX: 020D
CONSTANT: WM_MOUSELAST_4 HEX: 020A
CONSTANT: WM_MOUSELAST_PRE_4 HEX: 0209
CONSTANT: WM_PARENTNOTIFY HEX: 0210
CONSTANT: WM_ENTERMENULOOP HEX: 0211
CONSTANT: WM_EXITMENULOOP HEX: 0212
CONSTANT: WM_NEXTMENU HEX: 0213
CONSTANT: WM_SIZING HEX: 0214
CONSTANT: WM_CAPTURECHANGED HEX: 0215
CONSTANT: WM_MOVING HEX: 0216
CONSTANT: WM_POWERBROADCAST HEX: 0218
CONSTANT: WM_DEVICECHANGE HEX: 0219
CONSTANT: WM_MDICREATE HEX: 0220
CONSTANT: WM_MDIDESTROY HEX: 0221
CONSTANT: WM_MDIACTIVATE HEX: 0222
CONSTANT: WM_MDIRESTORE HEX: 0223
CONSTANT: WM_MDINEXT HEX: 0224
CONSTANT: WM_MDIMAXIMIZE HEX: 0225
CONSTANT: WM_MDITILE HEX: 0226
CONSTANT: WM_MDICASCADE HEX: 0227
CONSTANT: WM_MDIICONARRANGE HEX: 0228
CONSTANT: WM_MDIGETACTIVE HEX: 0229
CONSTANT: WM_MDISETMENU HEX: 0230
CONSTANT: WM_ENTERSIZEMOVE HEX: 0231
CONSTANT: WM_EXITSIZEMOVE HEX: 0232
CONSTANT: WM_DROPFILES HEX: 0233
CONSTANT: WM_MDIREFRESHMENU HEX: 0234
CONSTANT: WM_IME_SETCONTEXT HEX: 0281
CONSTANT: WM_IME_NOTIFY HEX: 0282
CONSTANT: WM_IME_CONTROL HEX: 0283
CONSTANT: WM_IME_COMPOSITIONFULL HEX: 0284
CONSTANT: WM_IME_SELECT HEX: 0285
CONSTANT: WM_IME_CHAR HEX: 0286
CONSTANT: WM_IME_REQUEST HEX: 0288
CONSTANT: WM_IME_KEYDOWN HEX: 0290
CONSTANT: WM_IME_KEYUP HEX: 0291
CONSTANT: WM_MOUSEHOVER HEX: 02A1
CONSTANT: WM_MOUSELEAVE HEX: 02A3
CONSTANT: WM_NCMOUSEHOVER HEX: 02A0
CONSTANT: WM_NCMOUSELEAVE HEX: 02A2
CONSTANT: WM_WTSSESSION_CHANGE HEX: 02B1
CONSTANT: WM_TABLET_FIRST HEX: 02c0
CONSTANT: WM_TABLET_LAST HEX: 02df
CONSTANT: WM_CUT HEX: 0300
CONSTANT: WM_COPY HEX: 0301
CONSTANT: WM_PASTE HEX: 0302
CONSTANT: WM_CLEAR HEX: 0303
CONSTANT: WM_UNDO HEX: 0304
CONSTANT: WM_RENDERFORMAT HEX: 0305
CONSTANT: WM_RENDERALLFORMATS HEX: 0306
CONSTANT: WM_DESTROYCLIPBOARD HEX: 0307
CONSTANT: WM_DRAWCLIPBOARD HEX: 0308
CONSTANT: WM_PAINTCLIPBOARD HEX: 0309
CONSTANT: WM_VSCROLLCLIPBOARD HEX: 030A
CONSTANT: WM_SIZECLIPBOARD HEX: 030B
CONSTANT: WM_ASKCBFORMATNAME HEX: 030C
CONSTANT: WM_CHANGECBCHAIN HEX: 030D
CONSTANT: WM_HSCROLLCLIPBOARD HEX: 030E
CONSTANT: WM_QUERYNEWPALETTE HEX: 030F
CONSTANT: WM_PALETTEISCHANGING HEX: 0310
CONSTANT: WM_PALETTECHANGED HEX: 0311
CONSTANT: WM_HOTKEY HEX: 0312
CONSTANT: WM_PRINT HEX: 0317
CONSTANT: WM_PRINTCLIENT HEX: 0318
CONSTANT: WM_APPCOMMAND HEX: 0319
CONSTANT: WM_THEMECHANGED HEX: 031A
CONSTANT: WM_HANDHELDFIRST HEX: 0358
CONSTANT: WM_HANDHELDLAST HEX: 035F
CONSTANT: WM_AFXFIRST HEX: 0360
CONSTANT: WM_AFXLAST HEX: 037F
CONSTANT: WM_PENWINFIRST HEX: 0380
CONSTANT: WM_PENWINLAST HEX: 038F
CONSTANT: WM_APP HEX: 8000
CONSTANT: WM_USER HEX: 0400
CONSTANT: EM_GETSEL HEX: 00B0
CONSTANT: EM_SETSEL HEX: 00B1
CONSTANT: EM_GETRECT HEX: 00B2
CONSTANT: EM_SETRECT HEX: 00B3
CONSTANT: EM_SETRECTNP HEX: 00B4
CONSTANT: EM_SCROLL HEX: 00B5
CONSTANT: EM_LINESCROLL HEX: 00B6
CONSTANT: EM_SCROLLCARET HEX: 00B7
CONSTANT: EM_GETMODIFY HEX: 00B8
CONSTANT: EM_SETMODIFY HEX: 00B9
CONSTANT: EM_GETLINECOUNT HEX: 00BA
CONSTANT: EM_LINEINDEX HEX: 00BB
CONSTANT: EM_SETHANDLE HEX: 00BC
CONSTANT: EM_GETHANDLE HEX: 00BD
CONSTANT: EM_GETTHUMB HEX: 00BE
CONSTANT: EM_LINELENGTH HEX: 00C1
CONSTANT: EM_REPLACESEL HEX: 00C2
CONSTANT: EM_GETLINE HEX: 00C4
CONSTANT: EM_LIMITTEXT HEX: 00C5
CONSTANT: EM_CANUNDO HEX: 00C6
CONSTANT: EM_UNDO HEX: 00C7
CONSTANT: EM_FMTLINES HEX: 00C8
CONSTANT: EM_LINEFROMCHAR HEX: 00C9
CONSTANT: EM_SETTABSTOPS HEX: 00CB
CONSTANT: EM_SETPASSWORDCHAR HEX: 00CC
CONSTANT: EM_EMPTYUNDOBUFFER HEX: 00CD
CONSTANT: EM_GETFIRSTVISIBLELINE HEX: 00CE
CONSTANT: EM_SETREADONLY HEX: 00CF
CONSTANT: EM_SETWORDBREAKPROC HEX: 00D0
CONSTANT: EM_GETWORDBREAKPROC HEX: 00D1
CONSTANT: EM_GETPASSWORDCHAR HEX: 00D2
CONSTANT: EM_SETMARGINS HEX: 00D3
CONSTANT: EM_GETMARGINS HEX: 00D4
ALIAS: EM_SETLIMITTEXT EM_LIMITTEXT
CONSTANT: EM_GETLIMITTEXT HEX: 00D5
CONSTANT: EM_POSFROMCHAR HEX: 00D6
CONSTANT: EM_CHARFROMPOS HEX: 00D7
CONSTANT: EM_SETIMESTATUS HEX: 00D8
CONSTANT: EM_GETIMESTATUS HEX: 00D9
CONSTANT: BM_GETCHECK HEX: 00F0
CONSTANT: BM_SETCHECK HEX: 00F1
CONSTANT: BM_GETSTATE HEX: 00F2
CONSTANT: BM_SETSTATE HEX: 00F3
CONSTANT: BM_SETSTYLE HEX: 00F4
CONSTANT: BM_CLICK HEX: 00F5
CONSTANT: BM_GETIMAGE HEX: 00F6
CONSTANT: BM_SETIMAGE HEX: 00F7
CONSTANT: STM_SETICON HEX: 0170
CONSTANT: STM_GETICON HEX: 0171
CONSTANT: STM_SETIMAGE HEX: 0172
CONSTANT: STM_GETIMAGE HEX: 0173
CONSTANT: STM_MSGMAX HEX: 0174
CONSTANT: DM_GETDEFID WM_USER
: DM_SETDEFID ( -- n ) ( -- n ) WM_USER 1  + ; inline
: DM_REPOSITION ( -- n ) ( -- n ) WM_USER 2  + ; inline
CONSTANT: LB_ADDSTRING HEX: 0180
CONSTANT: LB_INSERTSTRING HEX: 0181
CONSTANT: LB_DELETESTRING HEX: 0182
CONSTANT: LB_SELITEMRANGEEX HEX: 0183
CONSTANT: LB_RESETCONTENT HEX: 0184
CONSTANT: LB_SETSEL HEX: 0185
CONSTANT: LB_SETCURSEL HEX: 0186
CONSTANT: LB_GETSEL HEX: 0187
CONSTANT: LB_GETCURSEL HEX: 0188
CONSTANT: LB_GETTEXT HEX: 0189
CONSTANT: LB_GETTEXTLEN HEX: 018A
CONSTANT: LB_GETCOUNT HEX: 018B
CONSTANT: LB_SELECTSTRING HEX: 018C
CONSTANT: LB_DIR HEX: 018D
CONSTANT: LB_GETTOPINDEX HEX: 018E
CONSTANT: LB_FINDSTRING HEX: 018F
CONSTANT: LB_GETSELCOUNT HEX: 0190
CONSTANT: LB_GETSELITEMS HEX: 0191
CONSTANT: LB_SETTABSTOPS HEX: 0192
CONSTANT: LB_GETHORIZONTALEXTENT HEX: 0193
CONSTANT: LB_SETHORIZONTALEXTENT HEX: 0194
CONSTANT: LB_SETCOLUMNWIDTH HEX: 0195
CONSTANT: LB_ADDFILE HEX: 0196
CONSTANT: LB_SETTOPINDEX HEX: 0197
CONSTANT: LB_GETITEMRECT HEX: 0198
CONSTANT: LB_GETITEMDATA HEX: 0199
CONSTANT: LB_SETITEMDATA HEX: 019A
CONSTANT: LB_SELITEMRANGE HEX: 019B
CONSTANT: LB_SETANCHORINDEX HEX: 019C
CONSTANT: LB_GETANCHORINDEX HEX: 019D
CONSTANT: LB_SETCARETINDEX HEX: 019E
CONSTANT: LB_GETCARETINDEX HEX: 019F
CONSTANT: LB_SETITEMHEIGHT HEX: 01A0
CONSTANT: LB_GETITEMHEIGHT HEX: 01A1
CONSTANT: LB_FINDSTRINGEXACT HEX: 01A2
CONSTANT: LB_SETLOCALE HEX: 01A5
CONSTANT: LB_GETLOCALE HEX: 01A6
CONSTANT: LB_SETCOUNT HEX: 01A7
CONSTANT: LB_INITSTORAGE HEX: 01A8
CONSTANT: LB_ITEMFROMPOINT HEX: 01A9
CONSTANT: LB_MULTIPLEADDSTRING HEX: 01B1
CONSTANT: LB_GETLISTBOXINFO HEX: 01B2
CONSTANT: LB_MSGMAX_501 HEX: 01B3
CONSTANT: LB_MSGMAX_WCE4 HEX: 01B1
CONSTANT: LB_MSGMAX_4 HEX: 01B0
CONSTANT: LB_MSGMAX_PRE4 HEX: 01A8
CONSTANT: CB_GETEDITSEL HEX: 0140
CONSTANT: CB_LIMITTEXT HEX: 0141
CONSTANT: CB_SETEDITSEL HEX: 0142
CONSTANT: CB_ADDSTRING HEX: 0143
CONSTANT: CB_DELETESTRING HEX: 0144
CONSTANT: CB_DIR HEX: 0145
CONSTANT: CB_GETCOUNT HEX: 0146
CONSTANT: CB_GETCURSEL HEX: 0147
CONSTANT: CB_GETLBTEXT HEX: 0148
CONSTANT: CB_GETLBTEXTLEN HEX: 0149
CONSTANT: CB_INSERTSTRING HEX: 014A
CONSTANT: CB_RESETCONTENT HEX: 014B
CONSTANT: CB_FINDSTRING HEX: 014C
CONSTANT: CB_SELECTSTRING HEX: 014D
CONSTANT: CB_SETCURSEL HEX: 014E
CONSTANT: CB_SHOWDROPDOWN HEX: 014F
CONSTANT: CB_GETITEMDATA HEX: 0150
CONSTANT: CB_SETITEMDATA HEX: 0151
CONSTANT: CB_GETDROPPEDCONTROLRECT HEX: 0152
CONSTANT: CB_SETITEMHEIGHT HEX: 0153
CONSTANT: CB_GETITEMHEIGHT HEX: 0154
CONSTANT: CB_SETEXTENDEDUI HEX: 0155
CONSTANT: CB_GETEXTENDEDUI HEX: 0156
CONSTANT: CB_GETDROPPEDSTATE HEX: 0157
CONSTANT: CB_FINDSTRINGEXACT HEX: 0158
CONSTANT: CB_SETLOCALE HEX: 0159
CONSTANT: CB_GETLOCALE HEX: 015A
CONSTANT: CB_GETTOPINDEX HEX: 015B
CONSTANT: CB_SETTOPINDEX HEX: 015C
CONSTANT: CB_GETHORIZONTALEXTENT HEX: 015d
CONSTANT: CB_SETHORIZONTALEXTENT HEX: 015e
CONSTANT: CB_GETDROPPEDWIDTH HEX: 015f
CONSTANT: CB_SETDROPPEDWIDTH HEX: 0160
CONSTANT: CB_INITSTORAGE HEX: 0161
CONSTANT: CB_MULTIPLEADDSTRING HEX: 0163
CONSTANT: CB_GETCOMBOBOXINFO HEX: 0164
CONSTANT: CB_MSGMAX_501 HEX: 0165
CONSTANT: CB_MSGMAX_WCE400 HEX: 0163
CONSTANT: CB_MSGMAX_400 HEX: 0162
CONSTANT: CB_MSGMAX_PRE400 HEX: 015B
CONSTANT: SBM_SETPOS HEX: 00E0
CONSTANT: SBM_GETPOS HEX: 00E1
CONSTANT: SBM_SETRANGE HEX: 00E2
CONSTANT: SBM_SETRANGEREDRAW HEX: 00E6
CONSTANT: SBM_GETRANGE HEX: 00E3
CONSTANT: SBM_ENABLE_ARROWS HEX: 00E4
CONSTANT: SBM_SETSCROLLINFO HEX: 00E9
CONSTANT: SBM_GETSCROLLINFO HEX: 00EA
CONSTANT: SBM_GETSCROLLBARINFO HEX: 00EB
CONSTANT: LVM_FIRST HEX: 1000 ! ListView messages
CONSTANT: TV_FIRST HEX: 1100 ! TreeView messages
CONSTANT: HDM_FIRST HEX: 1200 ! Header messages
CONSTANT: TCM_FIRST HEX: 1300 ! Tab control messages
CONSTANT: PGM_FIRST HEX: 1400 ! Pager control messages
CONSTANT: ECM_FIRST HEX: 1500 ! Edit control messages
CONSTANT: BCM_FIRST HEX: 1600 ! Button control messages
CONSTANT: CBM_FIRST HEX: 1700 ! Combobox control messages
CONSTANT: CCM_FIRST HEX: 2000 ! Common control shared messages
: CCM_LAST ( -- n ) CCM_FIRST HEX: 0200  + ; inline
: CCM_SETBKCOLOR ( -- n ) CCM_FIRST  1  + ; inline
: CCM_SETCOLORSCHEME ( -- n ) CCM_FIRST  2  + ; inline
: CCM_GETCOLORSCHEME ( -- n ) CCM_FIRST  3  + ; inline
: CCM_GETDROPTARGET ( -- n ) CCM_FIRST  4  + ; inline
: CCM_SETUNICODEFORMAT ( -- n ) CCM_FIRST  5  + ; inline
: CCM_GETUNICODEFORMAT ( -- n ) CCM_FIRST  6  + ; inline
: CCM_SETVERSION ( -- n ) CCM_FIRST  7  + ; inline
: CCM_GETVERSION ( -- n ) CCM_FIRST  8  + ; inline
: CCM_SETNOTIFYWINDOW ( -- n ) CCM_FIRST  9  + ; inline
: CCM_SETWINDOWTHEME ( -- n ) CCM_FIRST  HEX: b  + ; inline
: CCM_DPISCALE ( -- n ) CCM_FIRST  HEX: c  + ; inline
: HDM_GETITEMCOUNT ( -- n ) HDM_FIRST  0  + ; inline
: HDM_INSERTITEMA ( -- n ) HDM_FIRST  1  + ; inline
: HDM_INSERTITEMW ( -- n ) HDM_FIRST  10  + ; inline
: HDM_DELETEITEM ( -- n ) HDM_FIRST  2  + ; inline
: HDM_GETITEMA ( -- n ) HDM_FIRST  3  + ; inline
: HDM_GETITEMW ( -- n ) HDM_FIRST  11  + ; inline
: HDM_SETITEMA ( -- n ) HDM_FIRST  4  + ; inline
: HDM_SETITEMW ( -- n ) HDM_FIRST  12  + ; inline
: HDM_LAYOUT ( -- n ) HDM_FIRST  5  + ; inline
: HDM_HITTEST ( -- n ) HDM_FIRST  6  + ; inline
: HDM_GETITEMRECT ( -- n ) HDM_FIRST  7  + ; inline
: HDM_SETIMAGELIST ( -- n ) HDM_FIRST  8  + ; inline
: HDM_GETIMAGELIST ( -- n ) HDM_FIRST  9  + ; inline
: HDM_ORDERTOINDEX ( -- n ) HDM_FIRST  15  + ; inline
: HDM_CREATEDRAGIMAGE ( -- n ) HDM_FIRST  16  + ; inline
: HDM_GETORDERARRAY ( -- n ) HDM_FIRST  17  + ; inline
: HDM_SETORDERARRAY ( -- n ) HDM_FIRST  18  + ; inline
: HDM_SETHOTDIVIDER ( -- n ) HDM_FIRST  19  + ; inline
: HDM_SETBITMAPMARGIN ( -- n ) HDM_FIRST  20  + ; inline
: HDM_GETBITMAPMARGIN ( -- n ) HDM_FIRST  21  + ; inline
CONSTANT: HDM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
CONSTANT: HDM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT   
: HDM_SETFILTERCHANGETIMEOUT ( -- n ) HDM_FIRST 22  + ; inline
: HDM_EDITFILTER ( -- n ) HDM_FIRST 23  + ; inline
: HDM_CLEARFILTER ( -- n ) HDM_FIRST 24  + ; inline
: TB_ENABLEBUTTON ( -- n ) WM_USER 1  + ; inline
: TB_CHECKBUTTON ( -- n ) WM_USER 2  + ; inline
: TB_PRESSBUTTON ( -- n ) WM_USER 3  + ; inline
: TB_HIDEBUTTON ( -- n ) WM_USER  4  + ; inline
: TB_INDETERMINATE ( -- n ) WM_USER  5  + ; inline
: TB_MARKBUTTON ( -- n ) WM_USER  6  + ; inline
: TB_ISBUTTONENABLED ( -- n ) WM_USER  9  + ; inline
: TB_ISBUTTONCHECKED ( -- n ) WM_USER  10  + ; inline
: TB_ISBUTTONPRESSED ( -- n ) WM_USER  11  + ; inline
: TB_ISBUTTONHIDDEN ( -- n ) WM_USER  12  + ; inline
: TB_ISBUTTONINDETERMINATE ( -- n ) WM_USER  13  + ; inline
: TB_ISBUTTONHIGHLIGHTED ( -- n ) WM_USER  14  + ; inline
: TB_SETSTATE ( -- n ) WM_USER  17  + ; inline
: TB_GETSTATE ( -- n ) WM_USER  18  + ; inline
: TB_ADDBITMAP ( -- n ) WM_USER  19  + ; inline
: TB_ADDBUTTONSA ( -- n ) WM_USER  20  + ; inline
: TB_INSERTBUTTONA ( -- n ) WM_USER  21  + ; inline
: TB_ADDBUTTONS ( -- n ) WM_USER  20  + ; inline
: TB_INSERTBUTTON ( -- n ) WM_USER  21  + ; inline
: TB_DELETEBUTTON ( -- n ) WM_USER  22  + ; inline
: TB_GETBUTTON ( -- n ) WM_USER  23  + ; inline
: TB_BUTTONCOUNT ( -- n ) WM_USER  24  + ; inline
: TB_COMMANDTOINDEX ( -- n ) WM_USER  25  + ; inline
: TB_SAVERESTOREA ( -- n ) WM_USER  26  + ; inline
: TB_SAVERESTOREW ( -- n ) WM_USER  76  + ; inline
: TB_CUSTOMIZE ( -- n ) WM_USER  27  + ; inline
: TB_ADDSTRINGA ( -- n ) WM_USER  28  + ; inline
: TB_ADDSTRINGW ( -- n ) WM_USER  77  + ; inline
: TB_GETITEMRECT ( -- n ) WM_USER  29  + ; inline
: TB_BUTTONSTRUCTSIZE ( -- n ) WM_USER  30  + ; inline
: TB_SETBUTTONSIZE ( -- n ) WM_USER  31  + ; inline
: TB_SETBITMAPSIZE ( -- n ) WM_USER  32  + ; inline
: TB_AUTOSIZE ( -- n ) WM_USER  33  + ; inline
: TB_GETTOOLTIPS ( -- n ) WM_USER  35  + ; inline
: TB_SETTOOLTIPS ( -- n ) WM_USER  36  + ; inline
: TB_SETPARENT ( -- n ) WM_USER  37  + ; inline
: TB_SETROWS ( -- n ) WM_USER  39  + ; inline
: TB_GETROWS ( -- n ) WM_USER  40  + ; inline
: TB_SETCMDID ( -- n ) WM_USER  42  + ; inline
: TB_CHANGEBITMAP ( -- n ) WM_USER  43  + ; inline
: TB_GETBITMAP ( -- n ) WM_USER  44  + ; inline
: TB_GETBUTTONTEXTA ( -- n ) WM_USER  45  + ; inline
: TB_GETBUTTONTEXTW ( -- n ) WM_USER  75  + ; inline
: TB_REPLACEBITMAP ( -- n ) WM_USER  46  + ; inline
: TB_SETINDENT ( -- n ) WM_USER  47  + ; inline
: TB_SETIMAGELIST ( -- n ) WM_USER  48  + ; inline
: TB_GETIMAGELIST ( -- n ) WM_USER  49  + ; inline
: TB_LOADIMAGES ( -- n ) WM_USER  50  + ; inline
: TB_GETRECT ( -- n ) WM_USER  51  + ; inline
: TB_SETHOTIMAGELIST ( -- n ) WM_USER  52  + ; inline
: TB_GETHOTIMAGELIST ( -- n ) WM_USER  53  + ; inline
: TB_SETDISABLEDIMAGELIST ( -- n ) WM_USER  54  + ; inline
: TB_GETDISABLEDIMAGELIST ( -- n ) WM_USER  55  + ; inline
: TB_SETSTYLE ( -- n ) WM_USER  56  + ; inline
: TB_GETSTYLE ( -- n ) WM_USER  57  + ; inline
: TB_GETBUTTONSIZE ( -- n ) WM_USER  58  + ; inline
: TB_SETBUTTONWIDTH ( -- n ) WM_USER  59  + ; inline
: TB_SETMAXTEXTROWS ( -- n ) WM_USER  60  + ; inline
: TB_GETTEXTROWS ( -- n ) WM_USER  61  + ; inline
: TB_GETOBJECT ( -- n ) WM_USER  62  + ; inline
: TB_GETHOTITEM ( -- n ) WM_USER  71  + ; inline
: TB_SETHOTITEM ( -- n ) WM_USER  72  + ; inline
: TB_SETANCHORHIGHLIGHT ( -- n ) WM_USER  73  + ; inline
: TB_GETANCHORHIGHLIGHT ( -- n ) WM_USER  74  + ; inline
: TB_MAPACCELERATORA ( -- n ) WM_USER  78  + ; inline
: TB_GETINSERTMARK ( -- n ) WM_USER  79  + ; inline
: TB_SETINSERTMARK ( -- n ) WM_USER  80  + ; inline
: TB_INSERTMARKHITTEST ( -- n ) WM_USER  81  + ; inline
: TB_MOVEBUTTON ( -- n ) WM_USER  82  + ; inline
: TB_GETMAXSIZE ( -- n ) WM_USER  83  + ; inline
: TB_SETEXTENDEDSTYLE ( -- n ) WM_USER  84  + ; inline
: TB_GETEXTENDEDSTYLE ( -- n ) WM_USER  85  + ; inline
: TB_GETPADDING ( -- n ) WM_USER  86  + ; inline
: TB_SETPADDING ( -- n ) WM_USER  87  + ; inline
: TB_SETINSERTMARKCOLOR ( -- n ) WM_USER  88  + ; inline
: TB_GETINSERTMARKCOLOR ( -- n ) WM_USER  89  + ; inline
ALIAS: TB_SETCOLORSCHEME CCM_SETCOLORSCHEME
ALIAS: TB_GETCOLORSCHEME CCM_GETCOLORSCHEME
ALIAS: TB_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: TB_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: TB_MAPACCELERATORW ( -- n ) WM_USER  90  + ; inline
: TB_GETBITMAPFLAGS ( -- n ) WM_USER  41  + ; inline
: TB_GETBUTTONINFOW ( -- n ) WM_USER  63  + ; inline
: TB_SETBUTTONINFOW ( -- n ) WM_USER  64  + ; inline
: TB_GETBUTTONINFOA ( -- n ) WM_USER  65  + ; inline
: TB_SETBUTTONINFOA ( -- n ) WM_USER  66  + ; inline
: TB_INSERTBUTTONW ( -- n ) WM_USER  67  + ; inline
: TB_ADDBUTTONSW ( -- n ) WM_USER  68  + ; inline
: TB_HITTEST ( -- n ) WM_USER  69  + ; inline
: TB_SETDRAWTEXTFLAGS ( -- n ) WM_USER  70  + ; inline
: TB_GETSTRINGW ( -- n ) WM_USER  91  + ; inline
: TB_GETSTRINGA ( -- n ) WM_USER  92  + ; inline
: TB_GETMETRICS ( -- n ) WM_USER  101  + ; inline
: TB_SETMETRICS ( -- n ) WM_USER  102  + ; inline
ALIAS: TB_SETWINDOWTHEME CCM_SETWINDOWTHEME
: RB_INSERTBANDA ( -- n ) WM_USER  1  + ; inline
: RB_DELETEBAND ( -- n ) WM_USER  2  + ; inline
: RB_GETBARINFO ( -- n ) WM_USER  3  + ; inline
: RB_SETBARINFO ( -- n ) WM_USER  4  + ; inline
: RB_GETBANDINFO ( -- n ) WM_USER  5  + ; inline
: RB_SETBANDINFOA ( -- n ) WM_USER  6  + ; inline
: RB_SETPARENT ( -- n ) WM_USER  7  + ; inline
: RB_HITTEST ( -- n ) WM_USER  8  + ; inline
: RB_GETRECT ( -- n ) WM_USER  9  + ; inline
: RB_INSERTBANDW ( -- n ) WM_USER  10  + ; inline
: RB_SETBANDINFOW ( -- n ) WM_USER  11  + ; inline
: RB_GETBANDCOUNT ( -- n ) WM_USER  12  + ; inline
: RB_GETROWCOUNT ( -- n ) WM_USER  13  + ; inline
: RB_GETROWHEIGHT ( -- n ) WM_USER  14  + ; inline
: RB_IDTOINDEX ( -- n ) WM_USER  16  + ; inline
: RB_GETTOOLTIPS ( -- n ) WM_USER  17  + ; inline
: RB_SETTOOLTIPS ( -- n ) WM_USER  18  + ; inline
: RB_SETBKCOLOR ( -- n ) WM_USER  19  + ; inline
: RB_GETBKCOLOR ( -- n ) WM_USER  20  + ; inline
: RB_SETTEXTCOLOR ( -- n ) WM_USER  21  + ; inline
: RB_GETTEXTCOLOR ( -- n ) WM_USER  22  + ; inline
: RB_SIZETORECT ( -- n ) WM_USER  23  + ; inline
CONSTANT: RB_SETCOLORSCHEME CCM_SETCOLORSCHEME
CONSTANT: RB_GETCOLORSCHEME CCM_GETCOLORSCHEME
: RB_BEGINDRAG ( -- n ) WM_USER  24  + ; inline
: RB_ENDDRAG ( -- n ) WM_USER  25  + ; inline
: RB_DRAGMOVE ( -- n ) WM_USER  26  + ; inline
: RB_GETBARHEIGHT ( -- n ) WM_USER  27  + ; inline
: RB_GETBANDINFOW ( -- n ) WM_USER  28  + ; inline
: RB_GETBANDINFOA ( -- n ) WM_USER  29  + ; inline
: RB_MINIMIZEBAND ( -- n ) WM_USER  30  + ; inline
: RB_MAXIMIZEBAND ( -- n ) WM_USER  31  + ; inline
ALIAS: RB_GETDROPTARGET CCM_GETDROPTARGET
: RB_GETBANDBORDERS ( -- n ) WM_USER  34  + ; inline
: RB_SHOWBAND ( -- n ) WM_USER  35  + ; inline
: RB_SETPALETTE ( -- n ) WM_USER  37  + ; inline
: RB_GETPALETTE ( -- n ) WM_USER  38  + ; inline
: RB_MOVEBAND ( -- n ) WM_USER  39  + ; inline
CONSTANT: RB_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
CONSTANT: RB_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: RB_GETBANDMARGINS ( -- n ) WM_USER  40  + ; inline
ALIAS: RB_SETWINDOWTHEME CCM_SETWINDOWTHEME
: RB_PUSHCHEVRON ( -- n ) WM_USER  43  + ; inline
: TTM_ACTIVATE ( -- n ) WM_USER  1  + ; inline
: TTM_SETDELAYTIME ( -- n ) WM_USER  3  + ; inline
: TTM_ADDTOOLA ( -- n ) WM_USER  4  + ; inline
: TTM_ADDTOOLW ( -- n ) WM_USER  50  + ; inline
: TTM_DELTOOLA ( -- n ) WM_USER  5  + ; inline
: TTM_DELTOOLW ( -- n ) WM_USER  51  + ; inline
: TTM_NEWTOOLRECTA ( -- n ) WM_USER  6  + ; inline
: TTM_NEWTOOLRECTW ( -- n ) WM_USER  52  + ; inline
: TTM_RELAYEVENT ( -- n ) WM_USER  7  + ; inline
: TTM_GETTOOLINFOA ( -- n ) WM_USER  8  + ; inline
: TTM_GETTOOLINFOW ( -- n ) WM_USER  53  + ; inline
: TTM_SETTOOLINFOA ( -- n ) WM_USER  9  + ; inline
: TTM_SETTOOLINFOW ( -- n ) WM_USER  54  + ; inline
: TTM_HITTESTA ( -- n ) WM_USER 10  + ; inline
: TTM_HITTESTW ( -- n ) WM_USER 55  + ; inline
: TTM_GETTEXTA ( -- n ) WM_USER 11  + ; inline
: TTM_GETTEXTW ( -- n ) WM_USER 56  + ; inline
: TTM_UPDATETIPTEXTA ( -- n ) WM_USER 12  + ; inline
: TTM_UPDATETIPTEXTW ( -- n ) WM_USER 57  + ; inline
: TTM_GETTOOLCOUNT ( -- n ) WM_USER 13  + ; inline
: TTM_ENUMTOOLSA ( -- n ) WM_USER 14  + ; inline
: TTM_ENUMTOOLSW ( -- n ) WM_USER 58  + ; inline
: TTM_GETCURRENTTOOLA ( -- n ) WM_USER  15  + ; inline
: TTM_GETCURRENTTOOLW ( -- n ) WM_USER  59  + ; inline
: TTM_WINDOWFROMPOINT ( -- n ) WM_USER  16  + ; inline
: TTM_TRACKACTIVATE ( -- n ) WM_USER  17  + ; inline
: TTM_TRACKPOSITION ( -- n ) WM_USER  18  + ; inline
: TTM_SETTIPBKCOLOR ( -- n ) WM_USER  19  + ; inline
: TTM_SETTIPTEXTCOLOR ( -- n ) WM_USER  20  + ; inline
: TTM_GETDELAYTIME ( -- n ) WM_USER  21  + ; inline
: TTM_GETTIPBKCOLOR ( -- n ) WM_USER  22  + ; inline
: TTM_GETTIPTEXTCOLOR ( -- n ) WM_USER  23  + ; inline
: TTM_SETMAXTIPWIDTH ( -- n ) WM_USER  24  + ; inline
: TTM_GETMAXTIPWIDTH ( -- n ) WM_USER  25  + ; inline
: TTM_SETMARGIN ( -- n ) WM_USER  26  + ; inline
: TTM_GETMARGIN ( -- n ) WM_USER  27  + ; inline
: TTM_POP ( -- n ) WM_USER  28  + ; inline
: TTM_UPDATE ( -- n ) WM_USER  29  + ; inline
: TTM_GETBUBBLESIZE ( -- n ) WM_USER  30  + ; inline
: TTM_ADJUSTRECT ( -- n ) WM_USER  31  + ; inline
: TTM_SETTITLEA ( -- n ) WM_USER  32  + ; inline
: TTM_SETTITLEW ( -- n ) WM_USER  33  + ; inline
: TTM_POPUP ( -- n ) WM_USER  34  + ; inline
: TTM_GETTITLE ( -- n ) WM_USER  35  + ; inline
ALIAS: TTM_SETWINDOWTHEME CCM_SETWINDOWTHEME
: SB_SETTEXTA ( -- n ) WM_USER 1 + ; inline
: SB_SETTEXTW ( -- n ) WM_USER 11  + ; inline
: SB_GETTEXTA ( -- n ) WM_USER 2  + ; inline
: SB_GETTEXTW ( -- n ) WM_USER 13  + ; inline
: SB_GETTEXTLENGTHA ( -- n ) WM_USER 3  + ; inline
: SB_GETTEXTLENGTHW ( -- n ) WM_USER 12  + ; inline
: SB_SETPARTS ( -- n ) WM_USER 4  + ; inline
: SB_GETPARTS ( -- n ) WM_USER 6  + ; inline
: SB_GETBORDERS ( -- n ) WM_USER 7  + ; inline
: SB_SETMINHEIGHT ( -- n ) WM_USER 8  + ; inline
: SB_SIMPLE ( -- n ) WM_USER 9  + ; inline
: SB_GETRECT ( -- n ) WM_USER 10  + ; inline
: SB_ISSIMPLE ( -- n ) WM_USER 14  + ; inline
: SB_SETICON ( -- n ) WM_USER 15  + ; inline
: SB_SETTIPTEXTA ( -- n ) WM_USER 16  + ; inline
: SB_SETTIPTEXTW ( -- n ) WM_USER 17  + ; inline
: SB_GETTIPTEXTA ( -- n ) WM_USER 18  + ; inline
: SB_GETTIPTEXTW ( -- n ) WM_USER 19  + ; inline
: SB_GETICON ( -- n ) WM_USER 20  + ; inline
CONSTANT: SB_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
CONSTANT: SB_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
ALIAS: SB_SETBKCOLOR CCM_SETBKCOLOR
CONSTANT: SB_SIMPLEID HEX: 00ff
ALIAS: TBM_GETPOS WM_USER
: TBM_GETRANGEMIN ( -- n ) WM_USER 1  + ; inline
: TBM_GETRANGEMAX ( -- n ) WM_USER 2  + ; inline
: TBM_GETTIC ( -- n ) WM_USER 3  + ; inline
: TBM_SETTIC ( -- n ) WM_USER 4  + ; inline
: TBM_SETPOS ( -- n ) WM_USER 5  + ; inline
: TBM_SETRANGE ( -- n ) WM_USER 6  + ; inline
: TBM_SETRANGEMIN ( -- n ) WM_USER 7  + ; inline
: TBM_SETRANGEMAX ( -- n ) WM_USER 8  + ; inline
: TBM_CLEARTICS ( -- n ) WM_USER 9  + ; inline
: TBM_SETSEL ( -- n ) WM_USER 10  + ; inline
: TBM_SETSELSTART ( -- n ) WM_USER 11  + ; inline
: TBM_SETSELEND ( -- n ) WM_USER 12  + ; inline
: TBM_GETPTICS ( -- n ) WM_USER 14  + ; inline
: TBM_GETTICPOS ( -- n ) WM_USER 15  + ; inline
: TBM_GETNUMTICS ( -- n ) WM_USER 16  + ; inline
: TBM_GETSELSTART ( -- n ) WM_USER 17  + ; inline
: TBM_GETSELEND ( -- n ) WM_USER 18  + ; inline
: TBM_CLEARSEL ( -- n ) WM_USER 19  + ; inline
: TBM_SETTICFREQ ( -- n ) WM_USER 20  + ; inline
: TBM_SETPAGESIZE ( -- n ) WM_USER 21  + ; inline
: TBM_GETPAGESIZE ( -- n ) WM_USER 22  + ; inline
: TBM_SETLINESIZE ( -- n ) WM_USER 23  + ; inline
: TBM_GETLINESIZE ( -- n ) WM_USER 24  + ; inline
: TBM_GETTHUMBRECT ( -- n ) WM_USER 25  + ; inline
: TBM_GETCHANNELRECT ( -- n ) WM_USER 26  + ; inline
: TBM_SETTHUMBLENGTH ( -- n ) WM_USER 27  + ; inline
: TBM_GETTHUMBLENGTH ( -- n ) WM_USER 28  + ; inline
: TBM_SETTOOLTIPS ( -- n ) WM_USER 29  + ; inline
: TBM_GETTOOLTIPS ( -- n ) WM_USER 30  + ; inline
: TBM_SETTIPSIDE ( -- n ) WM_USER 31  + ; inline
: TBM_SETBUDDY ( -- n ) WM_USER 32  + ; inline
: TBM_GETBUDDY ( -- n ) WM_USER 33  + ; inline
ALIAS: TBM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: TBM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: DL_BEGINDRAG ( -- n ) WM_USER 133  + ; inline
: DL_DRAGGING ( -- n ) WM_USER 134  + ; inline
: DL_DROPPED ( -- n ) WM_USER 135  + ; inline
: DL_CANCELDRAG ( -- n ) WM_USER 136  + ; inline
: UDM_SETRANGE ( -- n ) WM_USER 101  + ; inline
: UDM_GETRANGE ( -- n ) WM_USER 102  + ; inline
: UDM_SETPOS ( -- n ) WM_USER 103  + ; inline
: UDM_GETPOS ( -- n ) WM_USER 104  + ; inline
: UDM_SETBUDDY ( -- n ) WM_USER 105  + ; inline
: UDM_GETBUDDY ( -- n ) WM_USER 106  + ; inline
: UDM_SETACCEL ( -- n ) WM_USER 107  + ; inline
: UDM_GETACCEL ( -- n ) WM_USER 108  + ; inline
: UDM_SETBASE ( -- n ) WM_USER 109  + ; inline
: UDM_GETBASE ( -- n ) WM_USER 110  + ; inline
: UDM_SETRANGE32 ( -- n ) WM_USER 111  + ; inline
: UDM_GETRANGE32 ( -- n ) WM_USER 112  + ; inline
ALIAS: UDM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: UDM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: UDM_SETPOS32 ( -- n ) WM_USER 113  + ; inline
: UDM_GETPOS32 ( -- n ) WM_USER 114  + ; inline
: PBM_SETRANGE ( -- n ) WM_USER 1  + ; inline
: PBM_SETPOS ( -- n ) WM_USER 2  + ; inline
: PBM_DELTAPOS ( -- n ) WM_USER 3  + ; inline
: PBM_SETSTEP ( -- n ) WM_USER 4  + ; inline
: PBM_STEPIT ( -- n ) WM_USER 5  + ; inline
: PBM_SETRANGE32 ( -- n ) WM_USER 6  + ; inline
: PBM_GETRANGE ( -- n ) WM_USER 7  + ; inline
: PBM_GETPOS ( -- n ) WM_USER 8  + ; inline
: PBM_SETBARCOLOR ( -- n ) WM_USER 9  + ; inline
ALIAS: PBM_SETBKCOLOR CCM_SETBKCOLOR
: HKM_SETHOTKEY ( -- n ) WM_USER 1  + ; inline
: HKM_GETHOTKEY ( -- n ) WM_USER 2  + ; inline
: HKM_SETRULES ( -- n ) WM_USER 3  + ; inline
ALIAS: LVM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: LVM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: LVM_GETBKCOLOR ( -- n ) LVM_FIRST  0  + ; inline
: LVM_SETBKCOLOR ( -- n ) LVM_FIRST  1  + ; inline
: LVM_GETIMAGELIST ( -- n ) LVM_FIRST  2  + ; inline
: LVM_SETIMAGELIST ( -- n ) LVM_FIRST  3  + ; inline
: LVM_GETITEMCOUNT ( -- n ) LVM_FIRST  4  + ; inline
: LVM_GETITEMA ( -- n ) LVM_FIRST  5  + ; inline
: LVM_GETITEMW ( -- n ) LVM_FIRST  75  + ; inline
: LVM_SETITEMA ( -- n ) LVM_FIRST  6  + ; inline
: LVM_SETITEMW ( -- n ) LVM_FIRST  76  + ; inline
: LVM_INSERTITEMA ( -- n ) LVM_FIRST  7  + ; inline
: LVM_INSERTITEMW ( -- n ) LVM_FIRST  77  + ; inline
: LVM_DELETEITEM ( -- n ) LVM_FIRST  8  + ; inline
: LVM_DELETEALLITEMS ( -- n ) LVM_FIRST  9  + ; inline
: LVM_GETCALLBACKMASK ( -- n ) LVM_FIRST  10  + ; inline
: LVM_SETCALLBACKMASK ( -- n ) LVM_FIRST  11  + ; inline
: LVM_FINDITEMA ( -- n ) LVM_FIRST  13  + ; inline
: LVM_FINDITEMW ( -- n ) LVM_FIRST  83  + ; inline
: LVM_GETITEMRECT ( -- n ) LVM_FIRST  14  + ; inline
: LVM_SETITEMPOSITION ( -- n ) LVM_FIRST  15  + ; inline
: LVM_GETITEMPOSITION ( -- n ) LVM_FIRST  16  + ; inline
: LVM_GETSTRINGWIDTHA ( -- n ) LVM_FIRST  17  + ; inline
: LVM_GETSTRINGWIDTHW ( -- n ) LVM_FIRST  87  + ; inline
: LVM_HITTEST ( -- n ) LVM_FIRST  18  + ; inline
: LVM_ENSUREVISIBLE ( -- n ) LVM_FIRST  19  + ; inline
: LVM_SCROLL ( -- n ) LVM_FIRST  20  + ; inline
: LVM_REDRAWITEMS ( -- n ) LVM_FIRST  21  + ; inline
: LVM_ARRANGE ( -- n ) LVM_FIRST  22  + ; inline
: LVM_EDITLABELA ( -- n ) LVM_FIRST  23  + ; inline
: LVM_EDITLABELW ( -- n ) LVM_FIRST  118  + ; inline
: LVM_GETEDITCONTROL ( -- n ) LVM_FIRST  24  + ; inline
: LVM_GETCOLUMNA ( -- n ) LVM_FIRST  25  + ; inline
: LVM_GETCOLUMNW ( -- n ) LVM_FIRST  95  + ; inline
: LVM_SETCOLUMNA ( -- n ) LVM_FIRST  26  + ; inline
: LVM_SETCOLUMNW ( -- n ) LVM_FIRST  96  + ; inline
: LVM_INSERTCOLUMNA ( -- n ) LVM_FIRST  27  + ; inline
: LVM_INSERTCOLUMNW ( -- n ) LVM_FIRST  97  + ; inline
: LVM_DELETECOLUMN ( -- n ) LVM_FIRST  28  + ; inline
: LVM_GETCOLUMNWIDTH ( -- n ) LVM_FIRST  29  + ; inline
: LVM_SETCOLUMNWIDTH ( -- n ) LVM_FIRST  30  + ; inline
: LVM_CREATEDRAGIMAGE ( -- n ) LVM_FIRST  33  + ; inline
: LVM_GETVIEWRECT ( -- n ) LVM_FIRST  34  + ; inline
: LVM_GETTEXTCOLOR ( -- n ) LVM_FIRST  35  + ; inline
: LVM_SETTEXTCOLOR ( -- n ) LVM_FIRST  36  + ; inline
: LVM_GETTEXTBKCOLOR ( -- n ) LVM_FIRST  37  + ; inline
: LVM_SETTEXTBKCOLOR ( -- n ) LVM_FIRST  38  + ; inline
: LVM_GETTOPINDEX ( -- n ) LVM_FIRST  39  + ; inline
: LVM_GETCOUNTPERPAGE ( -- n ) LVM_FIRST  40  + ; inline
: LVM_GETORIGIN ( -- n ) LVM_FIRST  41  + ; inline
: LVM_UPDATE ( -- n ) LVM_FIRST  42  + ; inline
: LVM_SETITEMSTATE ( -- n ) LVM_FIRST  43  + ; inline
: LVM_GETITEMSTATE ( -- n ) LVM_FIRST  44  + ; inline
: LVM_GETITEMTEXTA ( -- n ) LVM_FIRST  45  + ; inline
: LVM_GETITEMTEXTW ( -- n ) LVM_FIRST  115  + ; inline
: LVM_SETITEMTEXTA ( -- n ) LVM_FIRST  46  + ; inline
: LVM_SETITEMTEXTW ( -- n ) LVM_FIRST  116  + ; inline
: LVM_SETITEMCOUNT ( -- n ) LVM_FIRST  47  + ; inline
: LVM_SORTITEMS ( -- n ) LVM_FIRST  48  + ; inline
: LVM_SETITEMPOSITION32 ( -- n ) LVM_FIRST  49  + ; inline
: LVM_GETSELECTEDCOUNT ( -- n ) LVM_FIRST  50  + ; inline
: LVM_GETITEMSPACING ( -- n ) LVM_FIRST  51  + ; inline
: LVM_GETISEARCHSTRINGA ( -- n ) LVM_FIRST  52  + ; inline
: LVM_GETISEARCHSTRINGW ( -- n ) LVM_FIRST  117  + ; inline
: LVM_SETICONSPACING ( -- n ) LVM_FIRST  53  + ; inline
: LVM_SETEXTENDEDLISTVIEWSTYLE ( -- n ) LVM_FIRST  54  + ; inline
: LVM_GETEXTENDEDLISTVIEWSTYLE ( -- n ) LVM_FIRST  55  + ; inline
: LVM_GETSUBITEMRECT ( -- n ) LVM_FIRST  56  + ; inline
: LVM_SUBITEMHITTEST ( -- n ) LVM_FIRST  57  + ; inline
: LVM_SETCOLUMNORDERARRAY ( -- n ) LVM_FIRST  58  + ; inline
: LVM_GETCOLUMNORDERARRAY ( -- n ) LVM_FIRST  59  + ; inline
: LVM_SETHOTITEM ( -- n ) LVM_FIRST  60  + ; inline
: LVM_GETHOTITEM ( -- n ) LVM_FIRST  61  + ; inline
: LVM_SETHOTCURSOR ( -- n ) LVM_FIRST  62  + ; inline
: LVM_GETHOTCURSOR ( -- n ) LVM_FIRST  63  + ; inline
: LVM_APPROXIMATEVIEWRECT ( -- n ) LVM_FIRST  64  + ; inline
: LVM_SETWORKAREAS ( -- n ) LVM_FIRST  65  + ; inline
: LVM_GETWORKAREAS ( -- n ) LVM_FIRST  70  + ; inline
: LVM_GETNUMBEROFWORKAREAS ( -- n ) LVM_FIRST  73  + ; inline
: LVM_GETSELECTIONMARK ( -- n ) LVM_FIRST  66  + ; inline
: LVM_SETSELECTIONMARK ( -- n ) LVM_FIRST  67  + ; inline
: LVM_SETHOVERTIME ( -- n ) LVM_FIRST  71  + ; inline
: LVM_GETHOVERTIME ( -- n ) LVM_FIRST  72  + ; inline
: LVM_SETTOOLTIPS ( -- n ) LVM_FIRST  74  + ; inline
: LVM_GETTOOLTIPS ( -- n ) LVM_FIRST  78  + ; inline
: LVM_SORTITEMSEX ( -- n ) LVM_FIRST  81  + ; inline
: LVM_SETBKIMAGEA ( -- n ) LVM_FIRST  68  + ; inline
: LVM_SETBKIMAGEW ( -- n ) LVM_FIRST  138  + ; inline
: LVM_GETBKIMAGEA ( -- n ) LVM_FIRST  69  + ; inline
: LVM_GETBKIMAGEW ( -- n ) LVM_FIRST  139  + ; inline
: LVM_SETSELECTEDCOLUMN ( -- n ) LVM_FIRST  140  + ; inline
: LVM_SETTILEWIDTH ( -- n ) LVM_FIRST  141  + ; inline
: LVM_SETVIEW ( -- n ) LVM_FIRST  142  + ; inline
: LVM_GETVIEW ( -- n ) LVM_FIRST  143  + ; inline
: LVM_INSERTGROUP ( -- n ) LVM_FIRST  145  + ; inline
: LVM_SETGROUPINFO ( -- n ) LVM_FIRST  147  + ; inline
: LVM_GETGROUPINFO ( -- n ) LVM_FIRST  149  + ; inline
: LVM_REMOVEGROUP ( -- n ) LVM_FIRST  150  + ; inline
: LVM_MOVEGROUP ( -- n ) LVM_FIRST  151  + ; inline
: LVM_MOVEITEMTOGROUP ( -- n ) LVM_FIRST  154  + ; inline
: LVM_SETGROUPMETRICS ( -- n ) LVM_FIRST  155  + ; inline
: LVM_GETGROUPMETRICS ( -- n ) LVM_FIRST  156  + ; inline
: LVM_ENABLEGROUPVIEW ( -- n ) LVM_FIRST  157  + ; inline
: LVM_SORTGROUPS ( -- n ) LVM_FIRST  158  + ; inline
: LVM_INSERTGROUPSORTED ( -- n ) LVM_FIRST  159  + ; inline
: LVM_REMOVEALLGROUPS ( -- n ) LVM_FIRST  160  + ; inline
: LVM_HASGROUP ( -- n ) LVM_FIRST  161  + ; inline
: LVM_SETTILEVIEWINFO ( -- n ) LVM_FIRST  162  + ; inline
: LVM_GETTILEVIEWINFO ( -- n ) LVM_FIRST  163  + ; inline
: LVM_SETTILEINFO ( -- n ) LVM_FIRST  164  + ; inline
: LVM_GETTILEINFO ( -- n ) LVM_FIRST  165  + ; inline
: LVM_SETINSERTMARK ( -- n ) LVM_FIRST  166  + ; inline
: LVM_GETINSERTMARK ( -- n ) LVM_FIRST  167  + ; inline
: LVM_INSERTMARKHITTEST ( -- n ) LVM_FIRST  168  + ; inline
: LVM_GETINSERTMARKRECT ( -- n ) LVM_FIRST  169  + ; inline
: LVM_SETINSERTMARKCOLOR ( -- n ) LVM_FIRST  170  + ; inline
: LVM_GETINSERTMARKCOLOR ( -- n ) LVM_FIRST  171  + ; inline
: LVM_SETINFOTIP ( -- n ) LVM_FIRST  173  + ; inline
: LVM_GETSELECTEDCOLUMN ( -- n ) LVM_FIRST  174  + ; inline
: LVM_ISGROUPVIEWENABLED ( -- n ) LVM_FIRST  175  + ; inline
: LVM_GETOUTLINECOLOR ( -- n ) LVM_FIRST  176  + ; inline
: LVM_SETOUTLINECOLOR ( -- n ) LVM_FIRST  177  + ; inline
: LVM_CANCELEDITLABEL ( -- n ) LVM_FIRST  179  + ; inline
: LVM_MAPINDEXTOID ( -- n ) LVM_FIRST  180  + ; inline
: LVM_MAPIDTOINDEX ( -- n ) LVM_FIRST  181  + ; inline
: TVM_INSERTITEMA ( -- n ) TV_FIRST  0  + ; inline
: TVM_INSERTITEMW ( -- n ) TV_FIRST  50  + ; inline
: TVM_DELETEITEM ( -- n ) TV_FIRST  1  + ; inline
: TVM_EXPAND ( -- n ) TV_FIRST  2  + ; inline
: TVM_GETITEMRECT ( -- n ) TV_FIRST  4  + ; inline
: TVM_GETCOUNT ( -- n ) TV_FIRST  5  + ; inline
: TVM_GETINDENT ( -- n ) TV_FIRST  6  + ; inline
: TVM_SETINDENT ( -- n ) TV_FIRST  7  + ; inline
: TVM_GETIMAGELIST ( -- n ) TV_FIRST  8  + ; inline
: TVM_SETIMAGELIST ( -- n ) TV_FIRST  9  + ; inline
: TVM_GETNEXTITEM ( -- n ) TV_FIRST  10  + ; inline
: TVM_SELECTITEM ( -- n ) TV_FIRST  11  + ; inline
: TVM_GETITEMA ( -- n ) TV_FIRST  12  + ; inline
: TVM_GETITEMW ( -- n ) TV_FIRST  62  + ; inline
: TVM_SETITEMA ( -- n ) TV_FIRST  13  + ; inline
: TVM_SETITEMW ( -- n ) TV_FIRST  63  + ; inline
: TVM_EDITLABELA ( -- n ) TV_FIRST  14  + ; inline
: TVM_EDITLABELW ( -- n ) TV_FIRST  65  + ; inline
: TVM_GETEDITCONTROL ( -- n ) TV_FIRST  15  + ; inline
: TVM_GETVISIBLECOUNT ( -- n ) TV_FIRST  16  + ; inline
: TVM_HITTEST ( -- n ) TV_FIRST  17  + ; inline
: TVM_CREATEDRAGIMAGE ( -- n ) TV_FIRST  18  + ; inline
: TVM_SORTCHILDREN ( -- n ) TV_FIRST  19  + ; inline
: TVM_ENSUREVISIBLE ( -- n ) TV_FIRST  20  + ; inline
: TVM_SORTCHILDRENCB ( -- n ) TV_FIRST  21  + ; inline
: TVM_ENDEDITLABELNOW ( -- n ) TV_FIRST  22  + ; inline
: TVM_GETISEARCHSTRINGA ( -- n ) TV_FIRST  23  + ; inline
: TVM_GETISEARCHSTRINGW ( -- n ) TV_FIRST  64  + ; inline
: TVM_SETTOOLTIPS ( -- n ) TV_FIRST  24  + ; inline
: TVM_GETTOOLTIPS ( -- n ) TV_FIRST  25  + ; inline
: TVM_SETINSERTMARK ( -- n ) TV_FIRST  26  + ; inline
ALIAS: TVM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: TVM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: TVM_SETITEMHEIGHT ( -- n ) TV_FIRST  27  + ; inline
: TVM_GETITEMHEIGHT ( -- n ) TV_FIRST  28  + ; inline
: TVM_SETBKCOLOR ( -- n ) TV_FIRST  29  + ; inline
: TVM_SETTEXTCOLOR ( -- n ) TV_FIRST  30  + ; inline
: TVM_GETBKCOLOR ( -- n ) TV_FIRST  31  + ; inline
: TVM_GETTEXTCOLOR ( -- n ) TV_FIRST  32  + ; inline
: TVM_SETSCROLLTIME ( -- n ) TV_FIRST  33  + ; inline
: TVM_GETSCROLLTIME ( -- n ) TV_FIRST  34  + ; inline
: TVM_SETINSERTMARKCOLOR ( -- n ) TV_FIRST  37  + ; inline
: TVM_GETINSERTMARKCOLOR ( -- n ) TV_FIRST  38  + ; inline
: TVM_GETITEMSTATE ( -- n ) TV_FIRST  39  + ; inline
: TVM_SETLINECOLOR ( -- n ) TV_FIRST  40  + ; inline
: TVM_GETLINECOLOR ( -- n ) TV_FIRST  41  + ; inline
: TVM_MAPACCIDTOHTREEITEM ( -- n ) TV_FIRST  42  + ; inline
: TVM_MAPHTREEITEMTOACCID ( -- n ) TV_FIRST  43  + ; inline
: CBEM_INSERTITEMA ( -- n ) WM_USER  1  + ; inline
: CBEM_SETIMAGELIST ( -- n ) WM_USER  2  + ; inline
: CBEM_GETIMAGELIST ( -- n ) WM_USER  3  + ; inline
: CBEM_GETITEMA ( -- n ) WM_USER  4  + ; inline
: CBEM_SETITEMA ( -- n ) WM_USER  5  + ; inline
ALIAS: CBEM_DELETEITEM CB_DELETESTRING
: CBEM_GETCOMBOCONTROL ( -- n ) WM_USER  6  + ; inline
: CBEM_GETEDITCONTROL ( -- n ) WM_USER  7  + ; inline
: CBEM_SETEXTENDEDSTYLE ( -- n ) WM_USER  14  + ; inline
: CBEM_GETEXTENDEDSTYLE ( -- n ) WM_USER  9  + ; inline
ALIAS: CBEM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: CBEM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: CBEM_SETEXSTYLE ( -- n ) WM_USER  8  + ; inline
: CBEM_GETEXSTYLE ( -- n ) WM_USER  9  + ; inline
: CBEM_HASEDITCHANGED ( -- n ) WM_USER  10  + ; inline
: CBEM_INSERTITEMW ( -- n ) WM_USER  11  + ; inline
: CBEM_SETITEMW ( -- n ) WM_USER  12  + ; inline
: CBEM_GETITEMW ( -- n ) WM_USER  13  + ; inline
: TCM_GETIMAGELIST ( -- n ) TCM_FIRST  2  + ; inline
: TCM_SETIMAGELIST ( -- n ) TCM_FIRST  3  + ; inline
: TCM_GETITEMCOUNT ( -- n ) TCM_FIRST  4  + ; inline
: TCM_GETITEMA ( -- n ) TCM_FIRST  5  + ; inline
: TCM_GETITEMW ( -- n ) TCM_FIRST  60  + ; inline
: TCM_SETITEMA ( -- n ) TCM_FIRST  6  + ; inline
: TCM_SETITEMW ( -- n ) TCM_FIRST  61  + ; inline
: TCM_INSERTITEMA ( -- n ) TCM_FIRST  7  + ; inline
: TCM_INSERTITEMW ( -- n ) TCM_FIRST  62  + ; inline
: TCM_DELETEITEM ( -- n ) TCM_FIRST  8  + ; inline
: TCM_DELETEALLITEMS ( -- n ) TCM_FIRST  9  + ; inline
: TCM_GETITEMRECT ( -- n ) TCM_FIRST  10  + ; inline
: TCM_GETCURSEL ( -- n ) TCM_FIRST  11  + ; inline
: TCM_SETCURSEL ( -- n ) TCM_FIRST  12  + ; inline
: TCM_HITTEST ( -- n ) TCM_FIRST  13  + ; inline
: TCM_SETITEMEXTRA ( -- n ) TCM_FIRST  14  + ; inline
: TCM_ADJUSTRECT ( -- n ) TCM_FIRST  40  + ; inline
: TCM_SETITEMSIZE ( -- n ) TCM_FIRST  41  + ; inline
: TCM_REMOVEIMAGE ( -- n ) TCM_FIRST  42  + ; inline
: TCM_SETPADDING ( -- n ) TCM_FIRST  43  + ; inline
: TCM_GETROWCOUNT ( -- n ) TCM_FIRST  44  + ; inline
: TCM_GETTOOLTIPS ( -- n ) TCM_FIRST  45  + ; inline
: TCM_SETTOOLTIPS ( -- n ) TCM_FIRST  46  + ; inline
: TCM_GETCURFOCUS ( -- n ) TCM_FIRST  47  + ; inline
: TCM_SETCURFOCUS ( -- n ) TCM_FIRST  48  + ; inline
: TCM_SETMINTABWIDTH ( -- n ) TCM_FIRST  49  + ; inline
: TCM_DESELECTALL ( -- n ) TCM_FIRST  50  + ; inline
: TCM_HIGHLIGHTITEM ( -- n ) TCM_FIRST  51  + ; inline
: TCM_SETEXTENDEDSTYLE ( -- n ) TCM_FIRST  52  + ; inline
: TCM_GETEXTENDEDSTYLE ( -- n ) TCM_FIRST  53  + ; inline
ALIAS: TCM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: TCM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
: ACM_OPENA ( -- n ) WM_USER 100  + ; inline
: ACM_OPENW ( -- n ) WM_USER 103  + ; inline
: ACM_PLAY ( -- n ) WM_USER 101  + ; inline
: ACM_STOP ( -- n ) WM_USER 102  + ; inline
CONSTANT: MCM_FIRST HEX: 1000
: MCM_GETCURSEL ( -- n ) MCM_FIRST  1  + ; inline
: MCM_SETCURSEL ( -- n ) MCM_FIRST  2  + ; inline
: MCM_GETMAXSELCOUNT ( -- n ) MCM_FIRST  3  + ; inline
: MCM_SETMAXSELCOUNT ( -- n ) MCM_FIRST  4  + ; inline
: MCM_GETSELRANGE ( -- n ) MCM_FIRST  5  + ; inline
: MCM_SETSELRANGE ( -- n ) MCM_FIRST  6  + ; inline
: MCM_GETMONTHRANGE ( -- n ) MCM_FIRST  7  + ; inline
: MCM_SETDAYSTATE ( -- n ) MCM_FIRST  8  + ; inline
: MCM_GETMINREQRECT ( -- n ) MCM_FIRST  9  + ; inline
: MCM_SETCOLOR ( -- n ) MCM_FIRST  10  + ; inline
: MCM_GETCOLOR ( -- n ) MCM_FIRST  11  + ; inline
: MCM_SETTODAY ( -- n ) MCM_FIRST  12  + ; inline
: MCM_GETTODAY ( -- n ) MCM_FIRST  13  + ; inline
: MCM_HITTEST ( -- n ) MCM_FIRST  14  + ; inline
: MCM_SETFIRSTDAYOFWEEK ( -- n ) MCM_FIRST  15  + ; inline
: MCM_GETFIRSTDAYOFWEEK ( -- n ) MCM_FIRST  16  + ; inline
: MCM_GETRANGE ( -- n ) MCM_FIRST  17  + ; inline
: MCM_SETRANGE ( -- n ) MCM_FIRST  18  + ; inline
: MCM_GETMONTHDELTA ( -- n ) MCM_FIRST  19  + ; inline
: MCM_SETMONTHDELTA ( -- n ) MCM_FIRST  20  + ; inline
: MCM_GETMAXTODAYWIDTH ( -- n ) MCM_FIRST  21  + ; inline
ALIAS: MCM_SETUNICODEFORMAT CCM_SETUNICODEFORMAT
ALIAS: MCM_GETUNICODEFORMAT CCM_GETUNICODEFORMAT
CONSTANT: DTM_FIRST HEX: 1000
: DTM_GETSYSTEMTIME ( -- n ) DTM_FIRST  1  + ; inline
: DTM_SETSYSTEMTIME ( -- n ) DTM_FIRST  2  + ; inline
: DTM_GETRANGE ( -- n ) DTM_FIRST  3  + ; inline
: DTM_SETRANGE ( -- n ) DTM_FIRST  4  + ; inline
: DTM_SETFORMATA ( -- n ) DTM_FIRST  5  + ; inline
: DTM_SETFORMATW ( -- n ) DTM_FIRST  50  + ; inline
: DTM_SETMCCOLOR ( -- n ) DTM_FIRST  6  + ; inline
: DTM_GETMCCOLOR ( -- n ) DTM_FIRST  7  + ; inline
: DTM_GETMONTHCAL ( -- n ) DTM_FIRST  8  + ; inline
: DTM_SETMCFONT ( -- n ) DTM_FIRST  9  + ; inline
: DTM_GETMCFONT ( -- n ) DTM_FIRST  10  + ; inline
: PGM_SETCHILD ( -- n ) PGM_FIRST  1  + ; inline
: PGM_RECALCSIZE ( -- n ) PGM_FIRST  2  + ; inline
: PGM_FORWARDMOUSE ( -- n ) PGM_FIRST  3  + ; inline
: PGM_SETBKCOLOR ( -- n ) PGM_FIRST  4  + ; inline
: PGM_GETBKCOLOR ( -- n ) PGM_FIRST  5  + ; inline
: PGM_SETBORDER ( -- n ) PGM_FIRST  6  + ; inline
: PGM_GETBORDER ( -- n ) PGM_FIRST  7  + ; inline
: PGM_SETPOS ( -- n ) PGM_FIRST  8  + ; inline
: PGM_GETPOS ( -- n ) PGM_FIRST  9  + ; inline
: PGM_SETBUTTONSIZE ( -- n ) PGM_FIRST  10  + ; inline
: PGM_GETBUTTONSIZE ( -- n ) PGM_FIRST  11  + ; inline
: PGM_GETBUTTONSTATE ( -- n ) PGM_FIRST  12  + ; inline
CONSTANT: PGM_GETDROPTARGET CCM_GETDROPTARGET
: BCM_GETIDEALSIZE ( -- n ) BCM_FIRST  1  + ; inline
: BCM_SETIMAGELIST ( -- n ) BCM_FIRST  2  + ; inline
: BCM_GETIMAGELIST ( -- n ) BCM_FIRST  3  + ; inline
: BCM_SETTEXTMARGIN ( -- n ) BCM_FIRST 4  + ; inline
: BCM_GETTEXTMARGIN ( -- n ) BCM_FIRST 5  + ; inline
: EM_SETCUEBANNER ( -- n ) ECM_FIRST  1 +  ; inline
: EM_GETCUEBANNER ( -- n ) ECM_FIRST  2 +  ; inline
: EM_SHOWBALLOONTIP ( -- n ) ECM_FIRST  3  + ; inline
: EM_HIDEBALLOONTIP ( -- n ) ECM_FIRST  4  + ; inline
: CB_SETMINVISIBLE ( -- n ) CBM_FIRST  1  + ; inline
: CB_GETMINVISIBLE ( -- n ) CBM_FIRST  2  + ; inline
: LM_HITTEST ( -- n ) WM_USER  HEX: 0300  + ; inline
: LM_GETIDEALHEIGHT ( -- n ) WM_USER  HEX: 0301  + ; inline
: LM_SETITEM ( -- n ) WM_USER  HEX: 0302  + ; inline
: LM_GETITEM ( -- n ) WM_USER  HEX: 0303  + ; inline



CONSTANT: WA_INACTIVE 0
CONSTANT: WA_ACTIVE 1
CONSTANT: WA_CLICKACTIVE 2

CONSTANT: SC_SIZE         HEX: f000
CONSTANT: SC_MOVE         HEX: f010
CONSTANT: SC_MINIMIZE     HEX: f020
CONSTANT: SC_MAXIMIZE     HEX: f030
CONSTANT: SC_NEXTWINDOW   HEX: f040
CONSTANT: SC_PREVWINDOW   HEX: f050
CONSTANT: SC_CLOSE        HEX: f060
CONSTANT: SC_VSCROLL      HEX: f070
CONSTANT: SC_HSCROLL      HEX: f080
CONSTANT: SC_MOUSEMENU    HEX: f090
CONSTANT: SC_KEYMENU      HEX: f100
CONSTANT: SC_ARRANGE      HEX: f110
CONSTANT: SC_RESTORE      HEX: f120
CONSTANT: SC_TASKLIST     HEX: f130
CONSTANT: SC_SCREENSAVE   HEX: f140
CONSTANT: SC_HOTKEY       HEX: f150

CONSTANT: APPCOMMAND_BROWSER_BACKWARD       1
CONSTANT: APPCOMMAND_BROWSER_FORWARD        2
CONSTANT: APPCOMMAND_BROWSER_REFRESH        3
CONSTANT: APPCOMMAND_BROWSER_STOP           4
CONSTANT: APPCOMMAND_BROWSER_SEARCH         5
CONSTANT: APPCOMMAND_BROWSER_FAVORITES      6
CONSTANT: APPCOMMAND_BROWSER_HOME           7
CONSTANT: APPCOMMAND_VOLUME_MUTE            8
CONSTANT: APPCOMMAND_VOLUME_DOWN            9
CONSTANT: APPCOMMAND_VOLUME_UP              10
CONSTANT: APPCOMMAND_MEDIA_NEXTTRACK        11
CONSTANT: APPCOMMAND_MEDIA_PREVIOUSTRACK    12
CONSTANT: APPCOMMAND_MEDIA_STOP             13
CONSTANT: APPCOMMAND_MEDIA_PLAY_PAUSE       14
CONSTANT: APPCOMMAND_LAUNCH_MAIL            15
CONSTANT: APPCOMMAND_LAUNCH_MEDIA_SELECT    16
CONSTANT: APPCOMMAND_LAUNCH_APP1            17
CONSTANT: APPCOMMAND_LAUNCH_APP2            18
CONSTANT: APPCOMMAND_BASS_DOWN              19
CONSTANT: APPCOMMAND_BASS_BOOST             20
CONSTANT: APPCOMMAND_BASS_UP                21
CONSTANT: APPCOMMAND_TREBLE_DOWN            22
CONSTANT: APPCOMMAND_TREBLE_UP              23
CONSTANT: APPCOMMAND_MICROPHONE_VOLUME_MUTE 24
CONSTANT: APPCOMMAND_MICROPHONE_VOLUME_DOWN 25
CONSTANT: APPCOMMAND_MICROPHONE_VOLUME_UP   26
CONSTANT: APPCOMMAND_HELP                   27
CONSTANT: APPCOMMAND_FIND                   28
CONSTANT: APPCOMMAND_NEW                    29
CONSTANT: APPCOMMAND_OPEN                   30
CONSTANT: APPCOMMAND_CLOSE                  31
CONSTANT: APPCOMMAND_SAVE                   32
CONSTANT: APPCOMMAND_PRINT                  33
CONSTANT: APPCOMMAND_UNDO                   34
CONSTANT: APPCOMMAND_REDO                   35
CONSTANT: APPCOMMAND_COPY                   36
CONSTANT: APPCOMMAND_CUT                    37
CONSTANT: APPCOMMAND_PASTE                  38
CONSTANT: APPCOMMAND_REPLY_TO_MAIL          39
CONSTANT: APPCOMMAND_FORWARD_MAIL           40
CONSTANT: APPCOMMAND_SEND_MAIL              41
CONSTANT: APPCOMMAND_SPELL_CHECK            42
CONSTANT: APPCOMMAND_DICTATE_OR_COMMAND_CONTROL_TOGGLE    43
CONSTANT: APPCOMMAND_MIC_ON_OFF_TOGGLE      44
CONSTANT: APPCOMMAND_CORRECTION_LIST        45
CONSTANT: APPCOMMAND_MEDIA_PLAY             46
CONSTANT: APPCOMMAND_MEDIA_PAUSE            47
CONSTANT: APPCOMMAND_MEDIA_RECORD           48
CONSTANT: APPCOMMAND_MEDIA_FAST_FORWARD     49
CONSTANT: APPCOMMAND_MEDIA_REWIND           50
CONSTANT: APPCOMMAND_MEDIA_CHANNEL_UP       51
CONSTANT: APPCOMMAND_MEDIA_CHANNEL_DOWN     52
CONSTANT: APPCOMMAND_DELETE                 53
CONSTANT: APPCOMMAND_DWM_FLIP3D             54

CONSTANT: FAPPCOMMAND_MOUSE HEX: 8000
CONSTANT: FAPPCOMMAND_KEY   HEX: 0000
CONSTANT: FAPPCOMMAND_OEM   HEX: 1000
CONSTANT: FAPPCOMMAND_MASK  HEX: F000
